# -*- coding: utf-8 -*-
"""
---------------------------------------------
Created on 2025/5/30 16:34
@author: ZhangYundi
@email: yundi.xxii@outlook.com
---------------------------------------------
"""

_BASE_LOCAL = "jydata"
TB_CALENDAR = f"{_BASE_LOCAL}/交易日历"
TB_SECU_MAIN = f"{_BASE_LOCAL}/证券主表"
TB_SHARES_INFO = f"{_BASE_LOCAL}/股本信息"
TB_LIST_STATUS = f"{_BASE_LOCAL}/上市信息"
TB_SPECIAL_TRADE = f"{_BASE_LOCAL}/特别处理"
TB_INDUSTRY_SW = f"{_BASE_LOCAL}/申万行业"
TB_ADJ_FACTOR = f"{_BASE_LOCAL}/复权因子"


_MC_LOCAL = "mc"
TB_STOCK_KLINE_DAY = f"{_MC_LOCAL}/stock_kline_day"
TB_STOCK_KLINE_MINUTE = f"{_MC_LOCAL}/stock_kline_minute"
TB_STOCK_TICK = f"{_MC_LOCAL}/stock_tick"
TB_INDEX_KLINE_DAY = f"{_MC_LOCAL}/index_kline_day"
TB_INDEX_KLINE_MINUTE = f"{_MC_LOCAL}/index_kline_minute"
